@@ -1,4 +1,5 @@
 import os
+import six
 import functools
 from os.path import join
 from mock import Mock, patch
@@ -18,13 +19,18 @@ from txtorcon import TorControlProtocol
 from txtorcon import TorProcessProtocol
 from txtorcon import launch
 from txtorcon import connect
-from txtorcon.controller import _is_non_public_numeric_address, Tor
+from txtorcon import AuthBasic
+from txtorcon.controller import _is_non_public_numeric_address, Tor, HAVE_ASYNC
 from txtorcon.interface import ITorControlProtocol
 from .util import TempDir
 
 from zope.interface import implementer, directlyProvides
 
 
+if HAVE_ASYNC:
+    from .py3_test_controller import ClientOnionServiceAuthenticationTests3  # noqa: F401
+
+
 class FakeProcessTransport(proto_helpers.StringTransportWithDisconnection):
     pid = -1
     reactor = None
@@ -146,6 +152,8 @@ class LaunchTorTests(unittest.TestCase):
             return defer.succeed('version=0.1.2.3')
         elif cmd == 'GETINFO events/names':
             return defer.succeed('events/names=STATUS_CLIENT')
+        elif cmd == 'GETINFO config/defaults':
+            return defer.succeed('config/defaults=')
         return defer.succeed(None)
 
     def _fake_event_listener(self, what, cb):
@@ -310,6 +318,22 @@ class LaunchTorTests(unittest.TestCase):
         ans = yield tor.dns_resolve("meejah.ca")
         self.assertEqual(ans, answer)
 
+    @patch('txtorcon.controller.socks')
+    @defer.inlineCallbacks
+    def test_dns_resolve_default_socksport(self, fake_socks):
+        answer = object()
+        cfg = Mock()
+        from txtorcon.testutil import FakeControlProtocol
+        proto = FakeControlProtocol([
+            {"SocksPort": "DEFAULT"},
+            "9050",
+        ])
+        proto.answers
+        tor = Tor(Mock(), proto, _tor_config=cfg)
+        fake_socks.resolve = Mock(return_value=defer.succeed(answer))
+        ans = yield tor.dns_resolve("meejah.ca")
+        self.assertEqual(ans, answer)
+
     @patch('txtorcon.controller.socks')
     @defer.inlineCallbacks
     def test_dns_resolve_existing_socks(self, fake_socks):
@@ -609,6 +633,8 @@ class LaunchTorTests(unittest.TestCase):
                     return defer.succeed('version=0.1.2.3')
                 elif cmd == 'GETINFO events/names':
                     return defer.succeed('events/names=STATUS_CLIENT')
+                elif cmd == 'GETINFO config/defaults':
+                    return defer.succeed('config/defaults=')
                 return defer.succeed(None)
             tpp.queue_command = fake_queue
             proto.makeConnection(Mock())
@@ -1047,9 +1073,10 @@ class TorAttributeTests(unittest.TestCase):
         self.tor = Tor(reactor, proto, _tor_config=self.cfg)
 
     def test_process(self):
-        with self.assertRaises(Exception) as ctx:
+        self.assertIs(
+            None,
             self.tor.process
-        self.assertTrue('not launched by us' in str(ctx.exception))
+        )
 
     def test_when_connected_already(self):
         tpp = TorProcessProtocol(lambda: None)
@@ -1090,6 +1117,48 @@ class TorAttributeTestsNoConfig(unittest.TestCase):
             self.assertEqual(gold, cfg)
 
 
+class DormantTests(unittest.TestCase):
+
+    def setUp(self):
+        reactor = Mock()
+        self.proto = Mock()
+        self.cfg = Mock()
+        self.tor = Tor(reactor, self.proto, _tor_config=self.cfg)
+        self.tor.dns_resolve = Mock()
+
+    @defer.inlineCallbacks
+    def test_ready(self):
+        self.proto.get_info = Mock(return_value={
+            "dormant": "0",
+            "status/enough-dir-info": "1",
+            "status/circuit-established": "1",
+        })
+        ready = yield self.tor.is_ready()
+        self.assertTrue(ready, "should be ready")
+
+    @defer.inlineCallbacks
+    def test_become_ready_already(self):
+        self.proto.get_info = Mock(return_value={
+            "dormant": "0",
+            "status/enough-dir-info": "1",
+            "status/circuit-established": "1",
+        })
+        yield self.tor.become_ready()
+        self.assertTrue(
+            self.tor.dns_resolve.mock_calls == []
+        )
+
+    @defer.inlineCallbacks
+    def test_become_ready_asleep(self):
+        self.proto.get_info = Mock(return_value={
+            "dormant": "1",
+            "status/enough-dir-info": "1",
+            "status/circuit-established": "1",
+        })
+        yield self.tor.become_ready()
+        self.assertEqual(1, len(self.tor.dns_resolve.mock_calls))
+
+
 class TorStreamTests(unittest.TestCase):
 
     def setUp(self):
@@ -1159,3 +1228,254 @@ class FactoryFunctionTests(unittest.TestCase):
         tor = Tor(Mock(), Mock())
         str(tor)
         # just testing the __str__ method doesn't explode
+
+
+class EphemeralOnionFactoryTests(unittest.TestCase):
+    """
+    the onion-service factory functions verify their args
+    """
+
+    def setUp(self):
+        reactor = Mock()
+        proto = Mock()
+        directlyProvides(proto, ITorControlProtocol)
+        self.cfg = Mock()
+        self.tor = Tor(reactor, proto, _tor_config=self.cfg)
+
+    @defer.inlineCallbacks
+    def test_ports_not_sequence(self):
+        with self.assertRaises(ValueError):
+            yield self.tor.create_onion_service("not a sequence")
+
+    @defer.inlineCallbacks
+    def test_ports_contain_non_ints(self):
+        with self.assertRaises(ValueError) as ctx:
+            yield self.tor.create_onion_service([object()])
+        self.assertIn("non-integer entry", str(ctx.exception))
+
+    @defer.inlineCallbacks
+    def test_ports_contain_non_ints2(self):
+        with self.assertRaises(ValueError) as ctx:
+            yield self.tor.create_onion_service([set([1, 2, 3])])
+        self.assertIn("contain a single int", str(ctx.exception))
+
+    @defer.inlineCallbacks
+    def test_ports_contain_non_ints3(self):
+        with self.assertRaises(ValueError) as ctx:
+            yield self.tor.create_onion_service([('not', 'an int')])
+        self.assertIn("non-integer", str(ctx.exception))
+
+    @defer.inlineCallbacks
+    def test_ports_contain_non_ints4(self):
+        with self.assertRaises(ValueError) as ctx:
+            yield self.tor.create_onion_service([('1234', 'bad')])
+        self.assertIn("be either an integer", str(ctx.exception))
+
+    @defer.inlineCallbacks
+    def test_ports_contain_non_ints5(self):
+        with self.assertRaises(ValueError) as ctx:
+            yield self.tor.create_onion_service([object()])
+        self.assertIn("non-integer entry", str(ctx.exception))
+
+    @defer.inlineCallbacks
+    def test_ports_contain_non_ints6(self):
+        from txtorcon.controller import _validate_ports
+        yield _validate_ports(Mock(), [80])
+
+    @defer.inlineCallbacks
+    def test_ports_contain_non_ints_unix_ok(self):
+        from txtorcon.controller import _validate_ports
+        yield _validate_ports(Mock(), [(80, "unix:/dev/null")])
+
+    @defer.inlineCallbacks
+    def test_ports_contain_2_tuple(self):
+        from txtorcon.controller import _validate_ports
+        yield _validate_ports(Mock(), [(80, 54321)])
+
+    @defer.inlineCallbacks
+    def test_ports_contain_string0(self):
+        from txtorcon.controller import _validate_ports
+        yield _validate_ports(Mock(), [u"80 127.0.0.1:1234"])
+
+    @defer.inlineCallbacks
+    def test_ports_contain_string1(self):
+        from txtorcon.controller import _validate_ports
+        yield _validate_ports(Mock(), ["80 127.0.0.1:1234"])
+
+    @defer.inlineCallbacks
+    def test_version_invalid(self):
+        with self.assertRaises(ValueError) as ctx:
+            yield self.tor.create_onion_service([80], version=1)
+        self.assertIn("The only valid Onion service versions", str(ctx.exception))
+
+    def test_auth(self):
+        self.tor.create_authenticated_onion_endpoint(80, AuthBasic(['alice']))
+
+    def test_auth_fs(self):
+        self.tor.create_filesystem_authenticated_onion_endpoint(80, '/dev/null', AuthBasic(['alice']))
+
+    @defer.inlineCallbacks
+    def test_happy_path(self):
+        self.cfg.EphemeralOnionServices = []
+        with patch('txtorcon.onion.available_tcp_port', return_value=1234):
+            with patch.object(self.cfg, 'tor_protocol') as proto:
+                proto.queue_command = Mock(return_value="ServiceID=deadbeef\nPrivateKey=BlobbyMcBlobberson")
+                d = self.tor.create_onion_service([80])
+                f = proto.add_event_listener.mock_calls[0][1][1]
+                f("UPLOAD deadbeef x dirauth0")
+                f("UPLOADED x x dirauth0")
+                service = yield d
+        self.assertEqual("deadbeef.onion", service.hostname)
+        self.assertEqual("BlobbyMcBlobberson", service.private_key)
+        self.assertEqual(set(['80 127.0.0.1:1234']), service.ports)
+
+
+class FilesystemOnionFactoryTests(unittest.TestCase):
+    """
+    the onion-service factory functions verify their args
+    """
+
+    def setUp(self):
+        reactor = Mock()
+        proto = Mock()
+        directlyProvides(proto, ITorControlProtocol)
+        self.cfg = Mock()
+        self.tor = Tor(reactor, proto, _tor_config=self.cfg)
+        self.hsdir = self.mktemp()
+        os.mkdir(self.hsdir)
+
+    @defer.inlineCallbacks
+    def test_ports_not_sequence(self):
+        with self.assertRaises(ValueError):
+            yield self.tor.create_filesystem_onion_service("not a sequence", self.hsdir)
+
+    @defer.inlineCallbacks
+    def test_ports_contain_non_ints0(self):
+        with self.assertRaises(ValueError) as ctx:
+            yield self.tor.create_filesystem_onion_service([('not', 'an int')], self.hsdir)
+        self.assertIn("a tuple with a non-integer", str(ctx.exception))
+
+    @defer.inlineCallbacks
+    def test_ports_contain_non_ints1(self):
+        with self.assertRaises(ValueError) as ctx:
+            yield self.tor.create_filesystem_onion_service([object()], self.hsdir)
+        self.assertIn("non-integer", str(ctx.exception))
+
+    @defer.inlineCallbacks
+    def test_version_invalid(self):
+        with self.assertRaises(ValueError) as ctx:
+            yield self.tor.create_filesystem_onion_service([80], self.hsdir, version=1)
+        self.assertIn("The only valid Onion service versions", str(ctx.exception))
+
+    @defer.inlineCallbacks
+    def test_happy_path(self):
+        self.cfg.OnionServices = []
+        with patch('txtorcon.onion.available_tcp_port', return_value=1234):
+            with patch.object(self.cfg, 'tor_protocol') as proto:
+                with open(join(self.hsdir, "hostname"), "w") as f:
+                    f.write("deadbeef.onion\n")
+                with open(join(self.hsdir, "hs_ed25519_secret_key"), "w") as f:
+                    f.write("BlobbyMcBlobberson")
+                proto.version = "0.3.2.1"
+                proto.queue_command = Mock(return_value="OK")
+                d = self.tor.create_filesystem_onion_service([80], self.hsdir)
+                f = proto.add_event_listener.mock_calls[0][1][1]
+                f("UPLOAD deadbeef x dirauth0")
+                f("UPLOADED x x dirauth0")
+                service = yield d
+        self.assertEqual("deadbeef.onion", service.hostname)
+        self.assertEqual(b"BlobbyMcBlobberson", service.private_key)
+        self.assertEqual(set(['80 127.0.0.1:1234']), set(service.ports))
+
+
+class FilesystemOnionEndpointFactoryTests(unittest.TestCase):
+
+    def setUp(self):
+        reactor = Mock()
+        proto = Mock()
+        directlyProvides(proto, ITorControlProtocol)
+        self.cfg = Mock()
+        self.tor = Tor(reactor, proto, _tor_config=self.cfg)
+
+    @defer.inlineCallbacks
+    def test_filesystem_endpoint(self):
+        yield self.tor.create_filesystem_onion_endpoint(80, '/dev/null')
+
+    @defer.inlineCallbacks
+    def test_ephemeral_endpoint(self):
+        yield self.tor.create_onion_endpoint(80)
+
+
+class ClientOnionServiceAuthenticationTests(unittest.TestCase):
+
+    def setUp(self):
+        reactor = Mock()
+        proto = Mock()
+        directlyProvides(proto, ITorControlProtocol)
+        self.cfg = TorConfig()
+        self.cfg.HidServAuth = ["existing.onion some_token"]
+        self.tor = Tor(reactor, proto, _tor_config=self.cfg)
+
+    @defer.inlineCallbacks
+    def test_add(self):
+        yield self.tor.add_onion_authentication("foo.onion", "a_token")
+        self.assertIn(
+            "foo.onion a_token",
+            self.cfg.HidServAuth,
+        )
+
+    @defer.inlineCallbacks
+    def test_add_twice(self):
+        yield self.tor.add_onion_authentication("foo.onion", "a_token")
+        self.assertIn(
+            "foo.onion a_token",
+            self.cfg.HidServAuth,
+        )
+        # a second add of the same token should be fine
+        yield self.tor.add_onion_authentication("foo.onion", "a_token")
+        self.assertIn(
+            "foo.onion a_token",
+            self.cfg.HidServAuth,
+        )
+
+    @defer.inlineCallbacks
+    def test_add_twice_different_token(self):
+        yield self.tor.add_onion_authentication("foo.onion", "a_token")
+        self.assertIn(
+            "foo.onion a_token",
+            self.cfg.HidServAuth,
+        )
+        # a second token with a different value: error
+        with self.assertRaises(ValueError):
+            yield self.tor.add_onion_authentication("foo.onion", "a_different_token")
+
+    @defer.inlineCallbacks
+    def test_remove(self):
+        yield self.tor.remove_onion_authentication("existing.onion")
+        self.assertEqual(0, len(self.cfg.HidServAuth))
+
+    @defer.inlineCallbacks
+    def test_remove_unfound(self):
+        yield self.tor.remove_onion_authentication("existing.onion")
+        self.assertEqual(0, len(self.cfg.HidServAuth))
+        yield self.tor.remove_onion_authentication("non_existing.onion")
+        self.assertEqual(0, len(self.cfg.HidServAuth))
+
+    def test_context_manager_py2(self):
+        if not six.PY2:
+            return
+        with self.assertRaises(RuntimeError):
+            self.tor.onion_authentication("foo.onion", "token")
+
+    @defer.inlineCallbacks
+    def test_add_and_remove(self):
+        yield self.tor.add_onion_authentication("foo.onion", "a_token")
+        self.assertIn(
+            "foo.onion a_token",
+            self.cfg.HidServAuth,
+        )
+        yield self.tor.remove_onion_authentication("foo.onion")
+        self.assertNotIn(
+            "foo.onion a_token",
+            self.cfg.HidServAuth,
+        )
