@@ -230,6 +230,28 @@ class DisconnectionTests(unittest.TestCase):
         self.protocol.connectionLost(f)
         self.assertTrue(it_was_called.yes)
 
+    def test_disconnect_outstanding_commands(self):
+        """
+        outstanding commands should errback on disconnect
+        """
+
+        def it_was_called(f):
+            str(f)
+            it_was_called.count += 1
+            return None
+        it_was_called.count = 0
+
+        # we want to make sure outstanding commands get errbacks
+        d0 = self.protocol.queue_command("some command0")
+        d1 = self.protocol.queue_command("some command1")
+        d0.addErrback(it_was_called)
+        d1.addErrback(it_was_called)
+        self.protocol.on_disconnect.addErrback(lambda _: None)
+
+        f = failure.Failure(RuntimeError("The thing didn't do the stuff."))
+        self.protocol.connectionLost(f)
+        self.assertEqual(it_was_called.count, 2)
+
 
 class ProtocolTests(unittest.TestCase):
 
@@ -615,6 +637,12 @@ OK'''.format(unexisting_file))
         self.send(b"250 ORPort=0")
         return d
 
+    def test_getconf_single(self):
+        d = self.protocol.get_conf_single("SOCKSPORT")
+        d.addCallback(CallbackChecker('9050'))
+        self.send(b"250 SocksPort=9050")
+        return d
+
     def response_ok(self, v):
         self.assertEqual(v, '')
 
@@ -741,6 +769,20 @@ OK'''.format(unexisting_file))
         )
         return d
 
+    def test_getinfo_single(self):
+        d = self.protocol.get_info_single("version")
+        d.addCallback(CallbackChecker('0.2.2.34'))
+        d.addErrback(self.fail)
+
+        self.send(b"250-version=0.2.2.34")
+        self.send(b"250 OK")
+
+        self.assertEqual(
+            self.transport.value(),
+            b"GETINFO version\r\n",
+        )
+        return d
+
     def test_getinfo_for_descriptor(self):
         descriptor_info = b"""250+desc/name/moria1=
 router moria1 128.31.0.34 9101 0 9131
@@ -828,7 +870,7 @@ platform Tor 0.2.5.0-alpha-dev on Linux
                 'SOMETHING_INVALID', lambda _: None
             )
             self.assertTrue(False)
-        except:
+        except Exception:
             pass
 
     def test_eventlistener(self):
